<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 01/09/2015
 * Time: 11:48
 */

namespace Controller;

use Exception\Messages;
use Service\LogService;
use Service\OAuth\OAuthCustomImpl;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

class ApiHcmServices extends Simple {

    /** @var OAuthCustomImpl $oauth */
    private $oauth;
    /** @var Kernel $kernel */
    private $kernel;
    /** @var \Service\ApiHcmServices $apiService */
    private $apiService;
    /** @var  \Service\ServiceLog */
    private $logService;

    public function __construct(OAuthCustomImpl $oauth, Kernel $kernel, \Service\ApiHcmServices $apiService, LogService $logService){
        $this->oauth = $oauth;
        $this->kernel = $kernel;
        $this->apiService = $apiService;
        $this->logService = $logService;
    }

    public function checkAccess(Request $request, Response $response){
        try{
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $httpRequest = $this->kernel->getHttpRequest();
            $token = $httpRequest->get('TOKEN');
            $clientSecret = $httpRequest->get('CLIENT_SECRET');
            $service = $this->oauth->checkAccess($token, $clientSecret);
            $response->addMessage(new Response\Message('Authorized', Response\Message::TYPE_MESSAGE));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_SPONSOR_ACCESS.'['.$e->getMessage().']', Response\Message::TYPE_ERROR));
        }
    }

    public function requestToken(Request $request, Response $response){
        try{
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $httpRequest = $this->kernel->getHttpRequest();
            $clientId = $httpRequest->get('CLIENT_ID');
            $clientSecret = $httpRequest->get('CLIENT_SECRET');
            $token = $this->oauth->grantAccessToken($clientId,$clientSecret);
            $response->addDataSet(new DataSet('token', array($token)));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message(Messages::ERR_SPONSOR_NOT_FOUND . $e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * @param $params       Array contendo os parametros obrigatórios e não obrigatórios.
     * @param $data         Array contendo os valores dos parametros a serem validados.
     * @return mixed        Retorna um array indexado numericamente com os valores dos parâmetros.
     * @throws \Exception   Caso exista um parâmetro obrigatório não informado retorna uma excessão informando o nome do parâmetro.
     */
    private function valida_parametros($params, $data)
    {
        $count = 0;
        foreach ($params as $param) {
            // Caso o parâmetro seja obrigatório e não esteja informado retorna um erro
            if ($param['req'] == 'S') {
                if (!isset($data[$param['name']])) {
                    throw new \Exception("Error occurred. Index ".$param['name']." is required.");
                }
            }
            // Retorna o valor do parâmetro
            if (isset($data[$param['name']])) {
                // Caso o valor do parâmetro seja uma string vazia retorna como nulo ou o valor padrão
                if ($data[$param['name']] == '') {
                    // Caso o parâmetro não seja obrigatório, não foi informado e possui valor default, é retornado o valor default
                    if ($param['req'] == 'N' && isset($param['default'])) {
                        $result[$count] = $param['default'];
                    } else {
                        $result[$count] = null;
                    }
                } else {
                    $result[$count] = $data[$param['name']];
                }
            } else {
                $result[$count] = null;
            }
            $count++;
        }
        return $result;
    }
    
    /*================================= GET FUNCTIONS =================================*/

    /**
     * Retorna todos os pagamentos de um determinado dia, feitos por um determinado cnpj em um determinado movimento de folha.
     *
     * @param Request\Row $request  No request são esperados os campos CNPJ, DATA E MOVIMENTO
     * - CNPJ           : Cnpj do empregador (somente números. ex: 01234567890001)
     * - DATA           : Data referente ao pagamento realizado (data no formado DD/MM/YYYY)
     * - MOVIMENTO      : Código do tipo de pagamento:
     *                      1-MENSAL;
     *                      2-FÉRIAS;
     *                      3-DÉCIMO TERCEIRO;
     *                      4-RESCISÃO;
     *                      5-ADIANTAMENTO;
     *                      6-PLR;
     *                      7-VALE TRANSPORTE;
     * - BANCO          : Banco da conta na qual o pagamento será debitado
     *                      104 - CAIXA ECONOMICA FEDERAL
     *                      033 - SANTANDER
     *                      356 - BANCO ABN AMRO
     *                      001 - BANCO DO BRASIL SA
     *                      237 - BRADESCO
     *                      756 - SICOOB
     *                      999 - BANCO FUNDO FIXO
     *                      341 - BANCO ITAU SA
     * - CDTPCTPAGAR    : Código do tipo de conta a pagar a ser filtrada na associação financeira
     * - CNPJFORNECEDOR : Cnpj do fornecedor a ser filtrado na associação financeira o qual receberá a sumarização dos pagamentos (somente números. ex: 01234567890001)
     * - DATACOMPETENCIA: Data da competência do pagamento realizado (data no formado DD/MM/YYYY)
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [pagamentos] = Array (
     *          [0] = Array (
     *              [cpf]    : Cpf do beneficiário
     *              [nome]   : Nome do beneficiário
     *              [data]   : Data de pagamento
     *              [contas] = Array (
     *                  [0] = Array (
     *                      [banco]  : Código do banco da conta para pagamento
     *                      [banco_nome]  : Nome do banco da conta para pagamento
     *                      [agencia]: Agencia da conta para pagamento
     *                      [conta]  : Número da conta para pagamento
     *                      [tipo]   : Tipo da conta para pagamento
     *                  )
     *              )
     *              [detEventos] = Array (
     *                  [Número do Evento] = [Valor Total do Evento]    // Ex: [3030] = 39614.15
     *              )
     *              [detVinculos] = Array(
     *                  [Número do vínculo] = Array(
     *                      [dtadmissaovinc]: Data de Admissão do Vínculo
     *                      [dtrescisaovinc]: Data de Rescisão do Vínculo
     *                      [eventos] = Array(
     *                          [0] = Array(
     *                              [dtcompetencia]: Data de Competência que é igual a Data de Ocorrência da FPA_CALCULOFOLHA
     *                              [dtocorrencia]: Data de Ocorrência do item de cálculo
     *                              [nreventom]: Número do Evento
     *                              [nrocorrecal]: Número da ocorrência do cálculo
     *                              [nrtipomoviment]: Número do Tipo de Movimento (MENSAL, FÉRIAS, DÉCIMO TERCEIRO E ETC)
     *                              [vreventofolha]: Valor do Evento
     *                          )
     *                      )
     *                      [nmvinculom]: Nome do Vínculo
     *                      [nrvinculom]: Número do Vínculo
     *                  )
     *              )
     *              [valor]  : Valor pago
     *              [centrocustos] : Array (
     *                  [codigo1] = [valor1]  : Código e valor referente ao centro de custo 1
     *                  [codigo2] = [valor2]  : Código e valor referente ao centro de custo 2
     *                  [codigoN] = [valorN]  : Código e valor referente ao centro de custo N
     *              )
     *              [filiais] : Array (
     *                  [centrocustos] : Array (
     *                      [codigo1] = [valor1]  : Código e valor referente ao centro de custo 1
     *                      [codigo2] = [valor2]  : Código e valor referente ao centro de custo 2
     *                      [codigoN] = [valorN]  : Código e valor referente ao centro de custo N
     *                  )
     *              )
     *          )
     *          [1] = ...
     *      )
     *  )
     * )
     */
    public function retornaPagamentosPorCnpjDataMovimento(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'CNPJ', 'req'=>'N'),
            1 => array('name'=>'DATA', 'req'=>'N'),
            2 => array('name'=>'MOVIMENTO', 'req'=>'S'),
            3 => array('name'=>'BANCO', 'req'=>'N'),
            4 => array('name'=>'CDTPCTPAGAR', 'req'=>'N'),
            5 => array('name'=>'CNPJFORNECEDOR', 'req'=>'N'),
            6 => array('name'=>'DATACOMPETENCIA', 'req'=>'S'),
        );
        try {
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->findItensPagamentoByCnpjAndDataAndMovimento($param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6]);
            //$this->logService->saveData($items);
            $response->addDataSet(new DataSet('pagamentos', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Retorna os resumos contábeis.
     *
     * @param Request\Row $request  No request são esperados os campos CNPJ E MES
     * - CNPJ       : Cnpj (somente números. ex: 01234567890001)
     * - MES        : Mes referente ao resumo contábil gerado (data no formado MM/YYYY)
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [lancamentos] = Array (
     *          [0] = Array (
     *              [id]          : Identificador do lançamento
     *              [num]         : Número do lançamento
     *              [tipo]        : Tipo do lançamento (Crédito/Débito)
     *              [data]        : Data do lançamento
     *              [cod_respons] : Código do responsável pelo lançamento
     *              [cod_unidade] : Código da unidade do responsável
     *              [conta]       : Código da Conta Contábil
     *              [desc_conta]  : Descrição da Conta Contábil
     *              [historico]   : Código do Histórico
     *              [desc_hist]   : Descrição do histórico
     *              [valor]       : Valor do lançamento
     *          )
     *          [1] = Array (
     *              [id]          : Identificador do lançamento
     *              [erro]        : Descrição do erro ocorrido
     *          )
     *          [2] = ...
     *      )
     *  )
     * )
     */
    public function retornaResumoContabilPorCnpjMes(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'CNPJ', 'req'=>'S'),
            1 => array('name'=>'MES', 'req'=>'S'),
        );
        try {
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->findResumoContabilByCnpjAndMes($param[0], $param[1]);
            $response->addDataSet(new DataSet('lancamentos', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Retorna todas as despesas de um determinado dia, feitos por um determinado cnpj em um determinado movimento de folha.
     *
     * @param Request\Row $request  No request são esperados os campos FILIAL, DATA E MOVIMENTO
     * - FILIAL     : Código da filial
     * - DATA       : Data referente à despesa (data no formado DD/MM/YYYY)
     * - MOVIMENTO  : Código do tipo de pagamento:
     *                  1-MENSAL;
     *                  2-FÉRIAS;
     *                  3-DÉCIMO TERCEIRO;
     *                  4-RESCISÃO;
     *                  5-ADIANTAMENTO;
     *                  6-PLR;
     *                  7-VALE TRANSPORTE;
     *                  null - MENSAL + DÉCIMO TERCEIRO
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [despesas] = Array (
     *          [0] = Array (
     *              [data]         : Data do lançamento
     *              [cod_respons]  : Código do responsável pelo lançamento
     *              [cod_despesa]  : Código da Despesa
     *              [desc_despesa] : Descrição da Despesa
     *              [valor]        : Valor do lançamento
     *              [qtd_func]     : Quantidade de funcionarios
     *          )
     *          [1] = Array (
     *              [id]          : Identificador do lançamento
     *              [erro]        : Descrição do erro ocorrido
     *          )
     *          [2] = ...
     *      )
     *  )
     * )
     */
    public function retornaResumoDespesaPorDataMovimento(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'FILIAL', 'req'=>'S'),
            1 => array('name'=>'DATA', 'req'=>'S'),
            2 => array('name'=>'MOVIMENTO', 'req'=>'S'),
        );
        try {
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->findResumoDroByCnpjAndDataAndMovimento($param[0], $param[1], $param[2]);
            $response->addDataSet(new DataSet('despesas', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função retorna lançamentos de cálculo conforme o filtro informado
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTMESCOMPETENC     : Data de competência do cálculo no formato DD/MM/YYYY
     * - NRTIPOMOVIMENT     : Movimento do cálculo. Caso não informado é utilizado 1-Mensal
     *                          1-MENSAL;
     *                          2-FÉRIAS;
     *                          3-DÉCIMO TERCEIRO;
     *                          4-RESCISÃO;
     *                          5-ADIANTAMENTO;
     *                          6-PLR;
     *                          7-VALE TRANSPORTE;
     * - NROCORRECAL        : Número da ocorrência de cálculo
     * - CDCNPJESTRUT       : Código de CNPJ à estrutura legal do vínculo
     * - CDFILIAL           : Código de filial referente à estrutura legal do vínculo
     * - CDCENTROCUSTO      : Código de centro de custo referente à estrutura gerencial do vínculo
     * - NRCPFPESSOA        : CPF da pessoa do vínculo (somente números)
     * - NREVENTOM          : Número do evento de folha do item ou lista de eventos separados por vírgulas sem espaços.
     *                          10 - Salário
     *                          70 - Hora Extra 50% ref. decimal
     *                          .
     *                          10,70,100,...
     * - DTOCORRENCIAITEM   : Data de lançamento do item no formato DD/MM/YYYY
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [itens] = Array (
     *          CDCNPJESTRUT        - CNPJ da estrutura legal do vínculo
     *          CDFILIAL            - Código de filial referente à estrutura legal do vínculo
     *          CDCENTROCUSTO       - Código de centro de custo referente à estrutura gerencial do vínculo
     *          NMTIPOMOVIMENT      - Nome do movimento (Mensal, Férias, 13 Salario)
     *          DTMESCOMPETENC      - Competência do movimento
     *          NMPESSOA            - Nome do vínculo
     *          NRCPFPESSOA         - CPF do vínculo
     *          NREVENTOM           - Número do evento de folha
     *          NMEVENTOH           - Nome do evento de folha
     *          IDTIPOEVENTO        - Identificador do tipo do evento
     *          IDUNIDREFEREVEN     - Identificador da unidade de referencia do evento
     *          VRPERCENTUALEVENTO  - Valor do percentual do evento
     *          VRREFFOLHA          - Valor de referencia do evento na folha
     *          VREVENTOFOLHA       - Valor do evento na folha
     *          DTOCORRENCIA        - Data de ocorrência do evento na folha
     *      )
     *  )
     * )
     */
    public function retornaLancamentosDeCalculo(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            1 => array('name'=>'NRTIPOMOVIMENT', 'req'=>'N', 'default' => '1'),
            2 => array('name'=>'NROCORRECAL', 'req'=>'N'),
            3 => array('name'=>'CDCNPJESTRUT', 'req'=>'N'),
            4 => array('name'=>'CDFILIAL', 'req'=>'N'),
            5 => array('name'=>'CDCENTROCUSTO', 'req'=>'N'),
            6 => array('name'=>'NRCPFPESSOA', 'req'=>'N'),
            7 => array('name'=>'NREVENTOM', 'req'=>'N'),
            8 => array('name'=>'DTOCORRENCIAITEM', 'req'=>'N'),
        );
        try {
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $result = $this->apiService->retornaLancamentosDeCalculo(
                $param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6], $param[7], $param[8]
            );
            $response->addDataSet(new DataSet('itens', $result));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função os dados referentes à pessoa: Pessoa mestre e histórico, parceiro de negócio, endereço, forma de comunicação e conta.
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTMESCOMPETENC     : Data de competência do cálculo no formato DD/MM/YYYY
     * - CDCNPJESTRUT       : Código de CNPJ à estrutura legal do vínculo
     * - CDFILIAL           : Código de filial referente à estrutura legal do vínculo
     * - CDCENTROCUSTO      : Código de centro de custo referente à estrutura gerencial do vínculo
     * - NRCPFPESSOA        : CPF da pessoa do vínculo (somente números)
     * - DTREFERENCIA       : Data de referência a partir da qual a pessoa foi incluída no formato DD/MM/YYYY
     * - CDINTEGOCUPA       : Código de integração da ocupação(cargo) do vínculo
     * - DTULTATU           : Data para buscar pessoas com alteração posterior a essa mesma data.
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [pessoas] = Array (
     *          [cpf] = Array (
     *              [gpe_pessoa] = Array ()     - Todos os dados da tabela gpe_pessoa
     *              [gpe_pessoah] = Array ()    - Todos os dados da tabela gpe_pessoah
     *              [parcnegocio] = Array ()    - Todos os dados da tabela parcnegocio
     *              [enderecoparc] = Array ()   - Todos os dados da tabela enderecoparc
     *              [comunicaparc] = Array ()   - Todos os dados da tabela comunicaparc
     *              [contcorrparc] = Array ()   - Todos os dados da tabela contcorrparc
     *              [gpe_vinculom] = Array ()   - Todos os dados da tabela gpe_vinculom
     *              [gpe_vinculoh] = Array ()   - Todos os dados da tabela gpe_vinculoh
     *              [gpe_ocupacao] = Array ()   - Todos os dados das tabelas gpe_ocupacaom e gpe_ocupacaoh
     *              [gpe_escala] = Array (      - Todos os dados das tabelas gpe_alteescala, gpe_escalatrabm e gpe_escalatrabh
     *                  ...
     *                  [gpe_turno] = Array ()      - Todos os dados das tabelas gpe_turno, gpe_hordiam e gpe_hordiah
     *              )
     *          )
     *      )
     *  )
     * )
     */
    public function retornaDadosPessoa(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            1 => array('name'=>'CDCNPJESTRUT', 'req'=>'N'),
            2 => array('name'=>'CDFILIAL', 'req'=>'N'),
            3 => array('name'=>'CDCENTROCUSTO', 'req'=>'N'),
            4 => array('name'=>'NRCPFPESSOA', 'req'=>'N'),
            5 => array('name'=>'DTREFERENCIA', 'req'=>'N'),
            6 => array('name'=>'CDINTEGOCUPA', 'req'=>'N'),
            7 => array('name'=>'DTULTATU', 'req'=>'N')
        );
        try {
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $result = $this->apiService->retornaDadosPessoa(
                $param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6], $param[7]
            );
            $response->addDataSet(new DataSet('pessoas', $result));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /**
     * Retorna todas as marcações de acordo com o périodo de tempo informado.
     *
     * @param Request\Row $request  No request são esperados os campos No request são esperados os campos:
     * Informações obrigatórias:
     * - NRCPFPESSOA    : CPF da pessoa do vínculo (somente números)
     * - DTINIPER : Data inicial do período filtrado (formato 'DD/MM/YYYY')
     * - DTFIMPER : Data final do período filtrado (formato 'DD/MM/YYYY')
     * Informações não obrigatórias(pelo menos uma delas deve ser informada):
     * - CDFILIAL       : Código de filial referente à estrutura legal do vínculo 
     * - CDCENTROCUSTO  : Código de centro de custo referente à estrutura gerencial do vínculo
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     Array (
     *  [dataset] = Array (
     *      [marcacaoponto] = Array (
     *          CDFILIAL                - Código de filial referente à estrutura legal do vínculo 
     *          CDCENTROCUSTO           - Código de centro de custo referente à estrutura gerencial do vínculo
     *          NRCPFPESSOA             - CPF da pessoa  
     *          DTHRMARCACAOPONTO       - Hora de marcação do ponto      
     *          DTHRAJUSTADA            - Hora ajustada do ponto   
     *          IDENQUADRAHORARIO       - Enquadramento de Horário('S'-> Sim 'N'->Não)       
     *          IDMARCDESPREZADA        - Marcação Desprezada('S'-> Sim 'N'->Não)       
     *          IDDIAOCORRENCIA         - Dia de Ocorrência:(
     *                                          "Dia anterior"->"DA"
     *                                          "Dia corrente"->"DC"
     *                                          "Dia seguinte"->"DS"
     *                                          "Força dia anterior"->"FA"
     *                                          "Força dia corrente"->"FC"
     *                                          "Força dia seguinte"->"FS")   
     *          NRCOLETOR               - Forma de Coleta de dados Ex: Digitada, Gerada pelo Sistema, Catraca
     *          IDFIXAENTSAID           -    
     *          IDREFERINTERVALO        -        
     *          IDNSRREP                -
     *          QTHRSDIURNAS            - Quantidades de Horas diurnas trabalhadas   
     *          QTHRSNOTURNAS           - Quantidades de Horas noturnas trabalhadas   
     *          QTHRSEXTRAS             - Quantidades de Horas extras 1
     *          PERCHRSEXTRAS           - Percentual do adicional de horas extras 1   
     *          IDTPHREXTRA             - Modalidade da hora extra 1, ("D" diurnas e "N" noturnas)
     *          QTHRSEXTRAS2            - Quantidades de Horas extras 2   
     *          PERCHRSEXTRAS2          - Percentual do adicional de horas extras 2   
     *          IDTPHREXTRA2            - Modalidade da hora extra 2, ("D" diurnas e "N" noturnas)   
     *          QTHRSEXTRAS3            - Quantidades de Horas extras 3   
     *          PERCHRSEXTRAS3          - Percentual do adicional de horas extras 3   
     *          IDTPHREXTRA3            - Modalidade da hora extra 3, ("D" diurnas e "N" noturnas)   
     *          QTHRSEXTRAS4            - Quantidades de Horas extras 4
     *          PERCHRSEXTRAS4          - Percentual do adicional de horas extras 4   
     *          IDTPHREXTRA4            - Modalidade da hora extra 4, ("D" diurnas e "N" noturnas)   
     *          QTHRSATRASOS            - Horas de faltas e/ou atrasos
     *          QTHRSCOMPENSAR          - Saldo de horas para compensar(positivo->banco de horas;negativo->horas devidas)   
     *          IDHRSCOMPENSAR          - Sinal de horas para compensar   
     *      )
     *  )
     * )
     */
    public function retornaMarcacoesPorCPF(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'NRCPFPESSOA', 'req'=>'N'),
            1 => array('name'=>'DTINIPER', 'req'=>'S'),
            2 => array('name'=>'DTFIMPER', 'req'=>'S'),
            3 => array('name'=>'CDFILIAL', 'req'=>'N'),
            4 => array('name'=>'CDCENTROCUSTO', 'req'=>'N'),
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->retornaMarcacoes($param[0], $param[1], $param[2],$param[3], $param[4]);
            $response->addDataSet(new DataSet('marcacaoponto', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna os dados referentes a alteracoes de escala
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTINICIO           : Filtro por data de inicio. Sao buscadas somente alteracoes com datas de inicio maiores que a informada
     * - DTFIM              : Filtro por data de fim. Sao buscadas somente alteracoes com datas de fim menores que a informada
     * - NRVINCULOM         : Filtro por numero de vinculo ou lista de numeros separados por virgula
     * - DTMESCOMPETENC     : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * - NRESTRUTLEGAL      : Filtro pelo número de estrutura legal
     * - CDCNPJTESTRUT      : Código de CNPJ à estrutura legal do vínculo
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [alteracoesEscala] = Array (
     *          [registration_number]   - Matricula ou numero do vinculo se a primeira nao estiver preenchida
     *          [initial_date]          - Data inicial da alteracao
     *          [shift_id]              - Cod. do turno da alteracao
     *          [shift_day_id]          - Numero do dia da semana da alteracao (Domingo: 1, Segunda: 2, Terca: 4, Quinta: 5, Sexta: 6, Sabado: 7)
     *          [...gpe_alteescala]     - Demais campos da tabela gpe_alteescala
     *      )
     *  )
     * )
     */
    public function getAlteracoesEscala(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTINICIO', 'req'=>'N'),
            1 => array('name'=>'DTFIM', 'req'=>'N'),
            2 => array('name'=>'NRVINCULOM', 'req'=>'N'),
            3 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            4 => array('name'=>'NRESTRUTLEGAL', 'req'=>'N'),
            5 => array('name'=>'CDCNPJESTRUT', 'req'=>'N'),
            6 => array('name'=>'NRPARCNEGOCIO', 'req'=>'N'),
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getAlteracoesEscala($param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6]);
            $response->addDataSet(new DataSet('alteracoesEscala', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna os dados referentes as estruturas
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - NRESTRUTURAM       : Filtro por numero de estrutura ou lista de numeros separados por virgula
     * - TIPO               : Tipo de estrutura (1 - Legal, 2 - Gerencial, 3 - Sindical). Se vazio, estrutura de todos os tipos sao consideradas
     * - NRTIPOESTRUTURA    : Tipo de estrutura conforme a tabela TIPOESTRUTURA
     * - NRTIPOESTRUTURASUP : Tipo da estrutura superior. Serao retornadas as estruturas do tipo informado que sao superiores as encontradas separadas por virgula
     * - DTMESCOMPETENC     : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [estruturas] = Array (
     *          [name]          - Nome do registro mestre da estrutura
     *          [code]          - Numero do registro mestre da estrutura
     *          [ESTRUTS_SUP]   - Lista de estruturas do tipo {NRTIPOESTRUTURASUP} hierarquicamente acima da estrutura
     *          [LEADERS_IDS]   - Matriculas (ou numeros de vinculos se vazias) dos responsaveis informais/adicionais pelas estruturas
     *          [...estruturam] - Demais campos da tabela estruturam
     *          [...estruturah] - Demais campos da tabela estruturah
     *      )
     *  )
     * )
     */
    public function getEstruturaPorTipo(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'NRESTRUTURAM', 'req'=>'N'),
            1 => array('name'=>'TIPO', 'req'=>'N'),
            2 => array('name'=>'NRTIPOESTRUTURA', 'req'=>'N'),
            3 => array('name'=>'NRTIPOESTRUTURASUP', 'req'=>'N'),
            4 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            5 => array('name'=> 'NRESTRUTURAMSUP', 'req'=>'N')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getEstruturaPorTipo($param[0], $param[1], $param[2], $param[3], $param[4], $param[5]);
            $response->addDataSet(new DataSet('estruturas', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna os dados referentes as estruturas do tipo CENTRO CUSTO
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - NRESTRUTURAM       : Filtro por numero de estrutura ou lista de numeros separados por virgula
     * - NRTIPOESTRUTURASUP : Tipo da estrutura superior. Serao retornadas as estruturas do tipo informado que sao superiores as encontradas separadas por virgula
     * - DTMESCOMPETENC     : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [estruturas] = Array (
     *          [name]          - Nome do registro mestre da estrutura
     *          [code]          - Numero do registro mestre da estrutura
     *          [ESTRUTS_SUP]   - Lista de estruturas do tipo {NRTIPOESTRUTURASUP} hierarquicamente acima da estrutura
     *          [LEADERS_IDS]   - Matriculas (ou numeros de vinculos se vazias) dos responsaveis informais/adicionais pelas estruturas
     *          [...estruturam] - Demais campos da tabela estruturam
     *          [...estruturah] - Demais campos da tabela estruturah
     *      )
     *  )
     * )
     */
    public function getCentrosDeCusto(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'NRESTRUTURAM', 'req'=>'N'),
            1 => array('name'=>'NRTIPOESTRUTURASUP', 'req'=>'N'),
            2 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            3 => array('name'=> 'NRESTRUTURAMSUP', 'req'=>'N')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getCentrosDeCusto($param[0], $param[1], $param[2], $param[3]);
            $response->addDataSet(new DataSet('estruturas', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna os dados referentes as estruturas do tipo DEPARTAMENTO
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - NRESTRUTURAM       : Filtro por numero de estrutura ou lista de numeros separados por virgula
     * - NRTIPOESTRUTURASUP : Tipo da estrutura superior. Serao retornadas as estruturas do tipo informado que sao superiores as encontradas separadas por virgula
     * - DTMESCOMPETENC     : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [estruturas] = Array (
     *          [name]              - Nome do registro mestre da estrutura
     *          [code]              - Numero do registro mestre da estrutura
     *          [business_unit_id]  - Numero das estruturas do tipo FILIAL superiores a estrutura
     *          [ESTRUTS_SUP]       - Lista de estruturas do tipo {NRTIPOESTRUTURASUP} hierarquicamente acima da estrutura
     *          [LEADERS_IDS]       - Matriculas (ou numeros de vinculos se vazias) dos responsaveis informais/adicionais pelas estruturas
     *          [...estruturam]     - Demais campos da tabela estruturam
     *          [...estruturah]     - Demais campos da tabela estruturah
     *      )
     *  )
     * )
     */
    public function getDepartamentos(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'NRESTRUTURAM', 'req'=>'N'),
            1 => array('name'=>'NRTIPOESTRUTURASUP', 'req'=>'N'),
            2 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            3 => array('name'=> 'NRESTRUTURAMSUP', 'req'=>'N')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getDepartamentos($param[0], $param[1], $param[2], $param[3]);
            $response->addDataSet(new DataSet('estruturas', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna os dados referentes as estruturas do tipo DEPARTAMENTO
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - NRESTRUTURAM       : Filtro por numero de estrutura ou lista de numeros separados por virgula
     * - NRTIPOESTRUTURASUP : Tipo da estrutura superior. Serao retornadas as estruturas do tipo informado que sao superiores as encontradas separadas por virgula
     * - DTMESCOMPETENC     : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [estruturas] = Array (
     *          [name]          - Nome do registro mestre da estrutura
     *          [code]          - Numero do registro mestre da estrutura
     *          [department_id] - Numero das estruturas do tipo DEPARTAMENTO superiores a estrutura
     *          [leader_ids]    - Mesmo valor de {ESTRUTS_SUP}
     *          [ESTRUTS_SUP]   - Lista de estruturas do tipo {NRTIPOESTRUTURASUP} hierarquicamente acima da estrutura
     *          [LEADERS_IDS]   - Matriculas (ou numeros de vinculos se vazias) dos responsaveis informais/adicionais pelas estruturas
     *          [...estruturam] - Demais campos da tabela estruturam
     *          [...estruturah] - Demais campos da tabela estruturah
     *      )
     *  )
     * )
     */
    public function getEquipes(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'NRESTRUTURAM', 'req'=>'N'),
            1 => array('name'=>'NRTIPOESTRUTURASUP', 'req'=>'N'),
            2 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            3 => array('name'=> 'NRESTRUTURAMSUP', 'req'=>'N')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getEquipes($param[0], $param[1], $param[2], $param[3]);
            $response->addDataSet(new DataSet('estruturas', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna os dados referentes as ocupacoes
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTMESCOMPETENC : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [cargos] = Array (
     *          [code]              - Cod. da ocupacao
     *          [name]              - Nome da ocupacao
     *          [female_name]       - Nome feminino da ocupacao
     *          [cbo]               - Codigo CBO (Classificacao Brasileira de Ocupacoes)
     *          [...gpe_ocupacaom]  - Demais campos da tabela gpe_ocupacaom
     *          [...gpe_ocupacaoh]  - Demais campos da tabela gpe_ocupacaoh
     *      )
     *  )
     * )
     */
    public function getCargos(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getCargos($param[0]);
            $response->addDataSet(new DataSet('cargos', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna os dados de vinculos demitidos
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTINICIO       : Filtro por data de inicio. Sao buscadas somente vinculos com datas de rescisao maiores que a informada
     * - DTFIM          : Filtro por data de fim. Sao buscadas somente vinculos com datas de rescisao menores que a informada
     * - NRVINCULOM     : Filtro por numero de vinculo ou lista de numeros separados por virgula
     * - DTMESCOMPETENC : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [demitidos] = Array (
     *          [employee_id]       - Matricula ou numero do vinculo se a primeira nao estiver preenchida
     *          [start_date]        - Data da rescisao
     *          [...gpe_vinculom]  - Demais campos da tabela gpe_vinculom
     *          [...gpe_vinculoh]  - Demais campos da tabela gpe_vinculoh
     *      )
     *  )
     * )
     */
    public function getDemitidos(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTINICIO', 'req'=>'N'),
            1 => array('name'=>'DTFIM', 'req'=>'N'),
            2 => array('name'=>'NRVINCULOM', 'req'=>'N'),
            3 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getDemitidos($param[0], $param[1], $param[2], $param[3]);
            $response->addDataSet(new DataSet('demitidos', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna os dados de vinculos admitidos ativos
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTINICIO       : Filtro por data de inicio. Sao buscadas somente vinculos com datas de admissao maiores que a informada
     * - DTFIM          : Filtro por data de fim. Sao buscadas somente vinculos com datas de admissao menores que a informada
     * - NRVINCULOM     : Filtro por numero de vinculo ou lista de numeros separados por virgula
     * - DTMESCOMPETENC : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * - NRESTRUTLEGAL  : Filtro pelo número de estrutura legal
     * - CDCNPJTESTRUT  : Código de CNPJ à estrutura legal do vínculo
     * - DTULTATU       : Data para buscar vínculos com alteração posterior a essa mesma data.
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [colaboradores] = Array (
     *          [registration_number]   - Matricula ou numero do vinculo se a primeira nao estiver preenchida
     *          [cpf]                   - CPF do colaborador
     *          [nis]                   - PIS do colaborador
     *          [job_title_id]          - Cod. do cargo
     *          [team_id]               - Cod. da estrutura gerencial
     *          [shift_id]              - Cod. do turno atual
     *          [cod_sind]              - Cod. da estrutura gerencial
     *          [admission_date]        - Data de admissao
     *          [is_clt]                - Se o colaborador tem vinculo CLT ou nao
     *          [...gpe_vinculom]       - Demais campos da tabela gpe_vinculom
     *          [...gpe_vinculoh]       - Demais campos da tabela gpe_vinculoh
     *          [...gpe_pessoah]        - Demais campos da tabela gpe_pessoah
     *          [...gpe_alteescala]     - Demais campos da tabela gpe_alteescala
     *      )
     *  )
     * )
     */
    public function getColaboradores(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTINICIO', 'req'=>'S'),
            1 => array('name'=>'DTFIM', 'req'=>'S'),
            2 => array('name'=>'NRVINCULOM', 'req'=>'N'),
            3 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            4 => array('name'=>'NRESTRUTLEGAL', 'req'=>'N'),
            5 => array('name'=>'CDCNPJESTRUT', 'req'=>'N'),
            6 => array('name'=>'DTULTATU', 'req'=>'N')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getColaboradores($param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6]);
            $response->addDataSet(new DataSet('colaboradores', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna as alteracoes de ocupacao
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTINICIO       : Filtro por data de inicio. Sao buscadas somente alteracoes com data de inicio maiores que a informada
     * - DTFIM          : Filtro por data de fim. Sao buscadas somente alteracoes com data de fim menores que a informada
     * - NRVINCULOM     : Filtro por numero de vinculo ou lista de numeros separados por virgula
     * - DTMESCOMPETENC : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [alteracoesOcupacao] = Array (
     *          [registration_number]   - Matricula ou numero do vinculo se a primeira nao estiver preenchida
     *          [initial_date]          - Data inicial da alteracao
     *          [shift_id]              - Cod. do cargo
     *          [...gpe_vinculom]       - Demais campos da tabela gpe_vinculom
     *          [...gpe_vinculoh]       - Demais campos da tabela gpe_vinculoh
     *          [...gpe_alteocupacao]   - Demais campos da tabela gpe_alteocupacao
     *      )
     *  )
     * )
     */
    public function getAlteracoesOcupacao(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTINICIO', 'req'=>'N'),
            1 => array('name'=>'DTFIM', 'req'=>'N'),
            2 => array('name'=>'NRVINCULOM', 'req'=>'N'),
            3 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getAlteracoesOcupacao($param[0], $param[1], $param[2], $param[3]);
            $response->addDataSet(new DataSet('alteracoesOcupacao', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna os afastamentos
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTINICIO       : Filtro por data de inicio. Sao buscados somente afastamentos com datas de inicio maiores que a informada
     * - DTFIM          : Filtro por data de fim. Sao buscados somente afastamentos com datas de fim menores que a informada
     * - NRVINCULOM     : Filtro por numero de vinculo ou lista de numeros separados por virgula
     * - DTMESCOMPETENC : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * - NRESTRUTLEGAL  : Filtro pelo número de estrutura legal
     * - CDCNPJTESTRUT  : Código de CNPJ à estrutura legal do vínculo
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [afastamentos] = Array (
     *          [start_date]            - Data inicio do afastamento
     *          [end_date]              - Data fim do afastamento
     *          [observation]           - Observacao do afastamento
     *          [absence_type]          - [id] 1; [name] Afastamento
     *          [is_medical_certificate]- Se o afastamento e um atestado medico
     *          [...gpe_vinculom]       - Demais campos da tabela gpe_vinculom
     *          [...gpe_vinculoh]       - Demais campos da tabela gpe_vinculoh
     *          [...gpe_altesitufunc]   - Demais campos da tabela gpe_altesitufunc
     *          [...gpe_situfunch]      - Demais campos da tabela gpe_situfunch
     *      )
     *  )
     * )
     */
    public function getAfastamentos(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTINICIO', 'req'=>'N'),
            1 => array('name'=>'DTFIM', 'req'=>'N'),
            2 => array('name'=>'NRVINCULOM', 'req'=>'N'),
            3 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            4 => array('name'=>'NRESTRUTLEGAL', 'req'=>'N'),
            5 => array('name'=>'CDCNPJESTRUT', 'req'=>'N'),
            6 => array('name'=>'NRPARCNEGOCIO', 'req'=>'N'),
            7 => array('name'=>'NRSITUFUNCM', 'req'=>'N')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getAfastamentos($param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6], $param[7]);
            $response->addDataSet(new DataSet('afastamentos', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função que retorna as ferias
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTINICIO       : Filtro por data de inicio. Sao buscados somente gozos de ferias com datas de inicio maiores que a informada
     * - DTFIM          : Filtro por data de fim. Sao buscados somente gozos de ferias com datas de fim menores que a informada
     * - NRVINCULOM     : Filtro por numero de vinculo ou lista de numeros separados por virgula
     * - DTMESCOMPETENC : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * - NRESTRUTLEGAL  : Filtro pelo número de estrutura legal
     * - CDCNPJTESTRUT  : Código de CNPJ à estrutura legal do vínculo
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [ferias] = Array (
     *          [start_date]            - Data inicio das ferias
     *          [end_date]              - Data fim das feias
     *          [observation]           - Observacao das ferias
     *          [absence_type]          - [id] 2; [name] Férias
     *          [is_medical_certificate]- Se o afastamento e um atestado medico
     *          [...gpe_vinculom]       - Demais campos da tabela gpe_vinculom
     *          [...gpe_vinculoh]       - Demais campos da tabela gpe_vinculoh
     *          [...fpa_ferias]         - Demais campos da tabela fpa_ferias
     *          [...fpa_gozoferias]     - Demais campos da tabela fpa_gozoferias
     *      )
     *  )
     * )
     */
    public function getFerias(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTINICIO', 'req'=>'N'),
            1 => array('name'=>'DTFIM', 'req'=>'N'),
            2 => array('name'=>'NRVINCULOM', 'req'=>'N'),
            3 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            4 => array('name'=>'NRESTRUTLEGAL', 'req'=>'N'),
            5 => array('name'=>'CDCNPJESTRUT', 'req'=>'N'),
            6 => array('name'=>'NRPARCNEGOCIO', 'req'=>'N'),
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getFerias($param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6]);
            $response->addDataSet(new DataSet('ferias', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /**
     * Função que retorna as movimentações por tipo de estrutura
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DTINICIO       : Filtro por data de inicio. Sao buscados somente movimentações com datas de inicio maiores que a informada
     * - DTFIM          : Filtro por data de fim. Sao buscados somente movimentações com datas de fim menores que a informada
     * - NUMTIPO        : Filtro por número do tipo de estrutura conforme informado abaixo:
     *      1- EMPRESA
     *      2- FILIAL
     *      3- DEPARTAMENTO
     *      4- SETOR
     *      5- CENTRO CUSTO
     *      6- EQUIPE
     *      7- TOMADOR DE SERVICOS
     *      8- REGIÃO
     *      9- PROJETO
     *      10- SINDICATO
     *      11- OBRA
     *      12- PRODUTO
     *      13- PLANO DE SAÚDE
     *      14- ESCOLA
     *      25- LABORATÓRIO
     *      26- FORNECEDOR
     * 
     * - DTMESCOMPETENC : Data de competencia. Se nao informada a competencia do mes atual e considerada
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [movimentacao] = Array (
     *          [registration_number]   - Código de mátricula ou número do vínculo
     *          [initial_date]          - Data início da movimentação
     *          [team_id]               - Código de integração da estrutura
     *          [cost_center_id]        - Código de integração da estrutura
     *          [job_title_id]          - Número do cargo
     *          [shift_id]              - Número do turno
     *          [cod_sind]              - Número da estrutura sindical
     *          [...gpe_vinculom]       - Demais campos da tabela gpe_vinculom
     *          [...gpe_vinculoh]       - Demais campos da tabela gpe_vinculoh
     *          [...gpe_alteescala]     - Demais campos da tabela gpe_alteescala
     *          [...gpe_ocupacaoh]      - Demais campos da tabela gpe_ocupacaoh
     *          [...gpe_movimentacao]   - Demais campos da tabela gpe_movimentacao
     *      )
     *  )
     * )
     */
    public function getMovimentacaoPorTipo(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DTINICIO', 'req'=>'N'),
            1 => array('name'=>'DTFIM', 'req'=>'N'),
            2 => array('name'=>'NUMTIPO', 'req'=>'N'),
            3 => array('name'=>'DTMESCOMPETENC', 'req'=>'N')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getMovimentacaoPorTipo($param[0], $param[1], $param[2], $param[3]);
            $response->addDataSet(new DataSet('movimentacao', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /**
     * Função destinada ao retorno de todas as situações funcionais do sistema HCM para o respectivo cliente.
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - dtReferencia       : Filtro por data de referência. Sao buscados somente situações funcionais cuja data de inclusão/alteração é igual ou posterior à data de referência passada como parâmetro (no formato DD/MM/YYYY).
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [getSituacaoFuncional] = Array (
     *          [CodSit]            - Código da situação
     *          [DesSit]            - Descrição da Situação
     *          [CodIntegracao]     - Código de Integração da Situação
     *
     *      )
     *  )
     * )
     */
    public function getSituacaoFuncional(Request\Row $request, Response $response) {
        $params = array(
            0 => array('name'=>'dtReferencia', 'req'=>'S')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getSituacaoFuncional($param[0]);

            $response->addDataSet(new DataSet('getSituacaoFuncional', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /**
     * Função destinada ao retorno das informações dos vínculos de acordo com o período de admissão
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - dtInicioAdmissao       : Data início para buscar vínculos dentro do período (no formato DD/MM/YYYY).
     * - dtFinalAdmissao        : Data final para buscar vínculos dentro do período (no formato DD/MM/YYYY).
     * - nrCodIntEstrutLegal    : Número do código de integração da estrutura legal.
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [getVinculosAdmissao] = Array (
     *          [eDatIni]           - Data Inicial para busca de admitidos
     *          [eDatFim]           - Data Final para busca de admitidos
     *          [eNumEmp]           - Número da Empresa para busca de admitidos
     *          [NumEmp]            - Código de Integração da estrutura legal do vínculo
     *          [CodFil]            - Código de Integração da estrutura legal do vínculo
     *          [NumCpf]            - Número do CPF (sem máscara)
     *          [NumCad]            - Matricula vínculo
     *          [MatEso]            - Matricula do eSocial
     *          [CodLoc]            - Código de Integração da estrutura gerencial
     *          [NomLoc]            - Nome da Estrutura Gerencial
     *          [CodCar]            - Código de Integração do Cargo
     *          [TitCar]            - Nome do Cargo
     *          [CodCb2]            - Número do CBO
     *          [NomFum]            - Nome do Vínculo
     *          [TipSex]            - Sexo do Colaborador M(Masculino) - F(Feminino) - O (Outros)
     *          [DatNas]            - Data Nascimento
     *          [DatAdm]            - Data de Admissão
     *          [SitAfa]            - Situação do Afastamento
     *
     *      )
     *  )
     * )
     */
    public function getVinculosAdmissao(Request\Row $request,Response $response) {
        $params = array(
            0 => array('name'=>'dtInicioAdmissao', 'req'=>'S'),
            1 => array('name'=>'dtFinalAdmissao', 'req'=>'S'),
            2 => array('name'=>'nrCodIntEstrutLegal', 'req'=>'S')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getVinculosAdmissao($param[0], $param[1], $param[2]);

            $response->addDataSet(new DataSet('getVinculosAdmissao', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /**
     * Função destinada ao retorno das informações dos vínculos afastados no período (no formato DD/MM/YYYY) do sistema HCM. 
     * O período de busca dos afastamentos deve contemplar afastamentos que iniciaram, terminaram ou estão entre o período selecionado. Como por exemplo:
     * Busca: 01/05/2023 a 05/05/2023
     * Retorna colaboradores que estão afastados neste período:
     * ex. Início: 01/04/2023 a Final: 03/05/2023;
     * ex. Início: 02/05/2023 a Final: 02/05/2023;
     * ex. Início: 03/05/2023 a Final: 07/05/2023;
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - dtInicio               : Data início para busca de afastamentos (no formato DD/MM/YYYY)
     * - dtFinal                : Data final para busca de afastamentos (no formato DD/MM/YYYY)
     * - nrCodIntEstrutLegal    : Número do código de integração da estrutura legal
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [getAfastamentoPorPeriodo] = Array (
     *          [eDatIni]       - Data Inicial para busca de afastamentos
     *          [eDatFim]       - Data Final para busca de afastamentos
     *          [eNumEmp]       - Número da Empresa para busca de admitidos
     *          [NumEmp]        - Código de Integração da estrutura legal do vínculo
     *          [NumCad]        - Matricula vínculo
     *          [NumCpf]        - Número do CPF (sem máscara)
     *          [SitAfa]        - Situação do Afastamento
     *          [DatAfa]        - Data do Início do Afastamento
     *          [DatTer]        - Data Final do Afastamento
     *          [...gpe_vinculom]       - Demais campos da tabela gpe_vinculom
     *          [...gpe_vinculoh]       - Demais campos da tabela gpe_vinculoh
     *          [...gpe_altesitufunc]   - Demais campos da tabela gpe_altesitufunc
     *          [...gpe_situfunch]      - Demais campos da tabela gpe_situfunch
     *      )
     *  )
     * )
     */
    public function getAfastamentoPorPeriodo(Request\Row $request, Response $response) {
        $params = array(
            0 => array('name'=>'dtInicio', 'req'=>'S'),
            1 => array('name'=>'dtFinal', 'req'=>'S'),
            2 => array('name'=>'nrCodIntEstrutLegal', 'req'=>'S')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getAfastamentoPorPeriodo($param[0], $param[1], $param[2]);

            $response->addDataSet(new DataSet('getAfastamentoPorPeriodo', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /**
     * Função destinada ao retorno das informações das movimentações ocorridas no período (no formato DD/MM/YYYY)
     * 
     * O período de busca deve contemplar movimentações que iniciaram, terminaramou estão entreoperíodo selecionado. Como por exemplo:
     * Busca: 01/05/2023 a 05/05/2023
     * Retorna movimentações neste período:
     * ex. Início: 01/04/2023 a Final: 03/05/2023;
     * ex. Início: 02/05/2023 a Final: 02/05/2023;
     * ex. Início: 03/05/2023 a Final: 07/05/2023;
     * ex. Início: 03/05/2023 a Final: vazio (movimentação em vigor);

     *
     * @param Request\Row $request  No request são esperados os campos:
     * - dtInicio               : Data início para busca de movimentação (no formato DD/MM/YYYY)
     * - dtFinal                : Data final para busca de movimentação (no formato DD/MM/YYYY)
     * - nrCodIntEstrutLegal    : Número do código de integração da estrutura legal
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [getMovimentacao] = Array (
     *          [eDatIni]       - Data Inicial para busca de movimentação
     *          [eDatFim]       - Data Final para busca de movimentação
     *          [eNumEmp]       - Número da Empresa para busca de admitidos
     *          [NumEmp]        - Código de Integração da estrutura legal do vínculo
     *          [NumCad]        - Matricula vínculo
     *          [NumCpf]        - Número do CPF (sem máscara)
     *          [MatEso]        - Matricula eSocial
     *          [CodFil]        - Código de Integração da estrutura legal do vínculo
     *          [CodCar]        - Código de Integração do Cargo
     *          [CodLoc]        - Código de Integração da Estrutura Gerencial
     *          [DatAlt]        - Data de Início da Movimentação
     *
     *      )
     *  )
     * )
     */
    public function getMovimentacao(Request\Row $request, Response $response) {
        $params = array(
            0 => array('name'=>'dtInicio', 'req'=>'S'),
            1 => array('name'=>'dtFinal', 'req'=>'S'),
            2 => array('name'=>'nrCodIntEstrutLegal', 'req'=>'S')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getMovimentacao($param[0], $param[1], $param[2]);

            $response->addDataSet(new DataSet('getMovimentacao', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /**
     * Função destinada ao retorno das informações de alteração dos cargos no período (no formato DD/MM/YYYY). 
     * 
     * O período de busca deve contemplar alterações de cargo que iniciaram, terminaramou estãoentreo período selecionado. Como por exemplo:
     * Busca: 01/05/2023 a 05/05/2023
     * Retorna alterações de cargo neste período:
     * ex. Início: 01/04/2023 a Final: 03/05/2023;
     * ex. Início: 02/05/2023 a Final: 02/05/2023;
     * ex. Início: 03/05/2023 a Final: 07/05/2023;
     * ex. Início: 03/05/2023 a Final: vazio (alteração de cargo em vigor);
     *
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - dtInicio               : Data início para busca de alteração de cargo (no formato DD/MM/YYYY)
     * - dtFinal                : Data final para busca de alteração de cargo (no formato DD/MM/YYYY)
     * - nrCodIntEstrutLegal    : Número do código de integração da estrutura legal
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [getMovimentacao] = Array (
     *          [eDatAlt]       - Data de Alteração para Busca do Cargo
     *          [eNumEmp]       - Número da Empresa para Busca do Cargo
     *          [NumEmp]        - Código de Integração da Estrutura Legal
     *          [NumCad]        - Matricula
     *          [NumCpf]        - Número do CPF
     *          [CodCar]        - Código de Integração do Cargo
     *          [DatAlt]        - Data de Início da Alteração
     *
     *      )
     *  )
     * )
     */
    public function getAlteracaoCargo(Request\Row $request, Response $response) {
        $params = array(
            0 => array('name'=>'dtInicio', 'req'=>'S'),
            1 => array('name'=>'dtFinal', 'req'=>'S'),
            2 => array('name'=>'nrCodIntEstrutLegal', 'req'=>'S')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getAlteracaoCargo($param[0], $param[1], $param[2]);

            $response->addDataSet(new DataSet('getAlteracaoCargo', $items));

        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /**
     * Função destinada ao retorno das informações de alteração de estrutura legal (empresa) no período(no formato DD/MM/YYYY). 
     * Tipo 1 --> Movimentação Legal (Empresa)
     * Tipo 2 --> Movimentação Gerencial (Local)
     * 
     * O período de busca deve contemplar movimentações que iniciaram, terminaramou estão entreoperíodo selecionado. Como por exemplo:
     * Busca: 01/05/2023 a 05/05/2023
     * Retorna movimentações neste período:
     * ex. Início: 01/04/2023 a Final: 03/05/2023;
     * ex. Início: 02/05/2023 a Final: 02/05/2023;
     * ex. Início: 03/05/2023 a Final: 07/05/2023;
     * ex. Início: 03/05/2023 a Final: vazio (movimentação em vigor);
     *
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - dtInicio               : Data início para busca de movimentação de estrutura legal (empresa) (no formato DD/MM/YYYY)
     * - dtFinal                : Data final para busca de movimentação de estrutura legal (empresa) (no formato DD/MM/YYYY)
     * - Tipo                   : Número do tipo de estrutura conforme informado abaixo:
     *      1 - Legal (empresa)
     *      2 - Gerencial (Local)
     * - nrCodIntEstrutLegal    : Número do código de integração da estrutura legal
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [getMovimentacao] = Array (
     *          [eDatAlt]       - Data de Alteração/Movimentação do Colaborador
     *          [eNumEmp]       - Número da Empresa para Busca da Movimentação
     *          [NumAtual]      - Código de Integração da Estrutura Atual
     *          [NumCad]        - Matricula
     *          [NumCpf]        - Número do CPF
     *          [NumAnterior]   - Código de Integração da Estrutura Anterior
     *          [DatAlt]        - Data de Alteração/Movimentação do colaborador na Empresa/Loca
     *          [Tipo]          - *      1 - Legal (empresa); 2 - Gerencial (Local)
     *
     *      )
     *  )
     * )
     */
    public function getMovporTipo(Request\Row $request,  Response $response) {
        $params = array(
            0 => array('name'=>'dtInicio', 'req'=>'S'),
            1 => array('name'=>'dtFinal', 'req'=>'S'),
            2 => array('name'=>'Tipo', 'req'=>'S'),
            3 => array('name'=>'nrCodIntEstrutLegal', 'req'=>'S')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getMovporTipo($param[0], $param[1], $param[2], $param[3]);

            $response->addDataSet(new DataSet('getMovporTipo', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    public function getTipoEstrutura(Request\Row $request, Response $response) {
        try {
            $items = $this->apiService->getTipoEstrutura();

            $response->addDataSet(new DataSet('getTipoEstrutura', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    public function getContraCheque(Request\Row $request,  Response $response) {
        $params = array(
            0 => array('name'=>'MATRICULAINICIAL', 'req'=>'N'),
            1 => array('name'=>'MATRICULAFINAL', 'req'=>'N'),
            2 => array('name'=>'NUMEROVINCULOINICIAL', 'req'=>'N'),
            3 => array('name'=>'NUMEROVINCULOFINAL', 'req'=>'N'),
            4 => array('name'=>'NOMEINICIALVINCULO', 'req'=>'N'),
            5 => array('name'=>'NOMEFINALVINCULO', 'req'=>'N'),
            6 => array('name'=>'ADMISSAOINICIAL', 'req'=>'N'),
            7 => array('name'=>'ADMISSAOFINAL', 'req'=>'N'),
            8 => array('name'=>'CNPJESTRUTURALEGAL', 'req'=>'N'),
            9 => array('name'=>'NUMEROESTRUTURALEGAL', 'req'=>'N'),
            10 => array('name'=>'COMPETENCIA', 'req'=>'S'),
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getContraCheque($param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6], $param[7], $param[8], $param[9], $param[10]);

            $response->addDataSet(new DataSet('getContraCheque', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    public function getMarcacaoPonto(Request\Row $request,  Response $response) {
        $params = array(
            0 => array('name'=>'DTMESCOMPETENC', 'req'=>'N'),
            1 => array('name'=>'NRVINCULOM', 'req'=>'N'),
            2 => array('name'=>'NRPARCNEGOCIO', 'req'=>'N'),
            3 => array('name'=>'NRTIPOVINCULOM', 'req'=>'N')
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $items = $this->apiService->getMarcacaoPonto($param[0], $param[1], $param[2], $param[3]);

            $response->addDataSet(new DataSet('getMarcacaoPonto', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    public function getTipoDemissao(Request\Row $request, Response $response) {
        try {
            $items = $this->apiService->getTipoDemissao();

            $response->addDataSet(new DataSet('getTipoDemissao', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    public function getMotivoRescisao(Request\Row $request, Response $response) {
        try {
            $items = $this->apiService->getMotivoRescisao();

            $response->addDataSet(new DataSet('getMotivoRescisao', $items));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /*=================================================================================*/
    
    /*================================= SET FUNCTIONS =================================*/
    
    /**
     * Função responsável por criar lotes e eventos e seus itens
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - DSLOTELANCEVENTO   : Descrição do Lote
     * - DTCOMPETLOTEEVE    : Data de competência do lote no formato DD/MM/YYYY
     * - NRTIPOMOVIMENT     : Movimento do lote. Caso não informado é utilizado 1-Mensal
     *                          1-MENSAL;
     *                          2-FÉRIAS;
     *                          3-DÉCIMO TERCEIRO;
     *                          4-RESCISÃO;
     *                          5-ADIANTAMENTO;
     *                          6-PLR;
     *                          7-VALE TRANSPORTE;
     * - DSOBSLOTELANC      : Observação do lote
     * - CDFILIAL           : Código de filial referente à estrutura legal do vínculo
     * - CDCENTROCUSTO      : Código de centro de custo referente à estrutura gerencial do vínculo
     * - NRCPFPESSOA        : CPF da pessoa do vínculo (somente números)
     * - NREVENTOM          : Número do evento de folha do item.
     *                          10 - Salário
     *                          .
     *                          .
     *                          .
     * - VRREFFOLHALOTE     : Valor de referência do item
     * - VREVENTOLOTE       : Valor do item
     * - DTOCORRENCIAITEM   : Data de lançamento do item no formato DD/MM/YYYY
     * - DSITEMLOTELANC     : Descrição do item
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [lote] = Array (
     *          true
     *      )
     *  )
     * )
     */
    public function setLancamentoLote(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'DSLOTELANCEVENTO', 'req'=>'S'),
            1 => array('name'=>'DTCOMPETLOTEEVE', 'req'=>'N'),
            2 => array('name'=>'NRTIPOMOVIMENT', 'req'=>'N', 'default' => '1'),
            3 => array('name'=>'DSOBSLOTELANC', 'req'=>'N'),
            4 => array('name'=>'CDFILIAL', 'req'=>'S'),
            5 => array('name'=>'CDCENTROCUSTO', 'req'=>'S'),
            6 => array('name'=>'NRCPFPESSOA', 'req'=>'S'),
            7 => array('name'=>'NREVENTOM', 'req'=>'S'),
            8 => array('name'=>'VRREFFOLHALOTE', 'req'=>'S'),
            9 => array('name'=>'VREVENTOLOTE', 'req'=>'S'),
            10 => array('name'=>'DTOCORRENCIAITEM', 'req'=>'N'),
            11 => array('name'=>'DSITEMLOTELANC', 'req'=>'N'),
        );
        try {
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $result = $this->apiService->setLancamentoLote(
                $param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6], $param[7], $param[8], $param[9], $param[10], $param[11]
            );
            $response->addDataSet(new DataSet('lote', array($result)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função responsável por criar movimentação gerencial para o vínculo
     *
     * @param Request\Row $request  No request são esperados os campos:
     * - CDFILIAL       : Código de filial referente à estrutura legal do vínculo (obrigatório)
     * - CDCENTROCUSTO  : Código de centro de custo referente à estrutura gerencial do vínculo (obrigatório)
     * - NRCPFPESSOA    : CPF da pessoa do vínculo (somente números) (obrigatório)
     * - DTINIMOVIMENT  : Data inicial da movimentação no formato DD/MM/YYYY (obrigatório)
     * - DTFIMMOVIMENT  : Data final da movimentação no formato DD/MM/YYYY
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [lote] = Array (
     *          true
     *      )
     *  )
     * )
     */
    public function setMovimentacaoGerencialVinculo(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'CDFILIAL', 'req'=>'S'),
            1 => array('name'=>'CDCENTROCUSTO', 'req'=>'S'),
            2 => array('name'=>'NRCPFPESSOA', 'req'=>'S'),
            3 => array('name'=>'DTINIMOVIMENT', 'req'=>'S'),
            4 => array('name'=>'DTFIMMOVIMENT', 'req'=>'N'),
        );
        try {
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $result = $this->apiService->setMovimentacaoGerencialVinculo(
                $param[0], $param[1], $param[2], $param[3], $param[4]
            );
            $response->addDataSet(new DataSet('setMovimentacaoGerencialVinculo', array($result)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função responsável por criar estrutura
     *
     * @param Request\Row $request  No request são esperados os campos:
     * Informações obrigatórias:
     * - NRTIPOESTRUTURA : Número do tipo de Estrutura conforme descrição abaixo:
     *      1- EMPRESA
     *      2- FILIAL
     *      3- DEPARTAMENTO
     *      4- SETOR
     *      5- CENTRO CUSTO
     *      6- EQUIPE
     *      7- TOMADOR DE SERVICOS
     *      8- REGIÃO
     *      9- PROJETO
     *      10- SINDICATO
     *      11- OBRA
     *      12- PRODUTO
     *      13- PLANO DE SAÚDE
     *      14- ESCOLA
     *      25- LABORATÓRIO
     *      26- FORNECEDOR
     * - DTINIVIGENCIA : Data de início de vigência da respectiva estrutura (caso não seja informada, ficará com a data 01/01/2000).
     * - NMFANTASIA: Nome fantasia do cliente.
     * - NMRAZSOCESTRUT: Razão social do cliente.
     * - CDCNPJESTRUT: Nº do CNPJ do Cliente.
     * - NRCEPENDERECO: CEP do endereço da estrutura.
     * - NMPAIS: Nome do país do endereço da estrutura.
     * - SGESTADO: Sigla do estado do endereço da estrutura.
     * - NMMUNICIPIO: Nome do município da estrutura.
     * - NMBAIRROENDERECO: Nome do bairro do endereço da estrutura.
     * - CDLOGRADOURO: Descrição do tipo de logradouro do endereço da estrutura (rua, avenida,…).
     * - DSENDERECO: Descrição do endereço da estrutura.
     * - NRIMOVELENDERECO: número da estrutura do endereço da estrutura.
     * - DSCOMPLEENDERECO: Descrição do complemento do endereço da estrutura.
     * - DSREFERENCIAENDE: Descrição da referência do endereço da estrutura.
     * Informações não obrigatórias:
     * - IDTIPOEMPR: Tipo de empresa conforme descrição abaixo:
     *      1 – Microempresa
     *      2 – Empresa de Pequeno Porte
     *      3 – Empresa/Órgão
     *      4 – Microempreendedor Individual
     *      5 – Empregador Doméstico
     *      6 – Empregador Rural
     * - IDOPTSIMPLES: Indica se a empresa é optante pelo simples (opções “S” ou “N”).
     * - IDPARTICIPAT: Indica se a empresa participa do “Programa de Alimentação do Trabalhador”.
     * - NRTIPOTRIBUT: tipo de tributação da empresa conforme opções abaixo:
     *      1 – Lucro Real
     *      2 – Lucro Presumido
     *      3 – Simples Nacional
     * - NRCLASSTRIBUT: código da classificação tributária (conforme tabela CLASSTRIBUT).
     * - VRALIQDESON: valor percentual da alíquota de desoneração.
     * - NRCALENDARIO: número do calendário da estrutura.
     * - CDCNAE: Código do CNAE da empresa.
     * - CDFPAS: Código do FPAS.
     * - CDTERCEIRO: código de terceiros.
     * - VRFAP: Valor do Fator Acidentário de Prevenção (FAP) da empresa.
     * - CDCEIESTRUT: CEI da empresa.
     * - NRCAGED: Código CAGED.
     * - CDEMPRCAIXA: Código de FGTS na Caixa Econômica Federal.
     * - CDFEDERALESTRUT: Código do registro federal (exemplo: código ANS do plano de saúde).
     * - NRPROPRIEMPR: CPF do proprietário.
     * - NRPARCRESPINF: CPF do responsável informal.
     * - NRPARCENTISIN: CPF do responsável legal.
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [estrutura] = Array (
     *          true
     *      )
     *  )
     * )
     */
    public function setEstrutura(Request\Row $request, Response $response){
        $params = array(
            0 => array('name'=>'NRTIPOESTRUTURA', 'req'=>'S'),
            1 => array('name'=>'DTINIVIGENCIA', 'req'=>'S'),
            2 => array('name'=>'NMFANTASIA', 'req'=>'S'),
            3 => array('name'=>'NMRAZSOCESTRUT', 'req'=>'S'),
            4 => array('name'=>'CDCNPJESTRUT', 'req'=>'S'),
            5 => array('name'=>'NRCEPENDERECO', 'req'=>'S'),
            6 => array('name'=>'NMPAIS', 'req'=>'S'),
            7 => array('name'=>'SGESTADO', 'req'=>'S'),
            8 => array('name'=>'NMMUNICIPIO', 'req'=>'S'),
            9 => array('name'=>'NMBAIRROENDERECO', 'req'=>'S'),
            10 => array('name'=>'CDLOGRADOURO', 'req'=>'S'),
            11 => array('name'=>'DSENDERECO', 'req'=>'S'),
            12 => array('name'=>'NRIMOVELENDERECO', 'req'=>'S'),
            13 => array('name'=>'DSCOMPLEENDERECO', 'req'=>'S'),
            14 => array('name'=>'DSREFERENCIAENDE', 'req'=>'S'),
            15 => array('name'=>'IDTIPOEMPR', 'req'=>'N'),
            16 => array('name'=>'IDOPTSIMPLES', 'req'=>'N'),
            17 => array('name'=>'IDPARTICIPAT', 'req'=>'N'),
            18 => array('name'=>'NRTIPOTRIBUT', 'req'=>'N'),
            19 => array('name'=>'NRCLASSTRIBUT', 'req'=>'N'),
            20 => array('name'=>'VRALIQDESON', 'req'=>'N'),
            21 => array('name'=>'NRCALENDARIO', 'req'=>'N'),
            22 => array('name'=>'CDCNAE', 'req'=>'N'),
            23 => array('name'=>'CDFPAS', 'req'=>'N'),
            24 => array('name'=>'CDTERCEIRO', 'req'=>'N'),
            25 => array('name'=>'VRFAP', 'req'=>'N'),
            26 => array('name'=>'CDCEIESTRUT', 'req'=>'N'),
            27 => array('name'=>'NRCAGED', 'req'=>'N'),
            28 => array('name'=>'CDEMPRCAIXA', 'req'=>'N'),
            29 => array('name'=>'CDFEDERALESTRUT', 'req'=>'N'),
            30 => array('name'=>'NRPROPRIEMPR', 'req'=>'N'),
            31 => array('name'=>'NRPARCRESPINF', 'req'=>'N'),
            32 => array('name'=>'NRPARCENTISIN', 'req'=>'N'),
            33 => array('name'=>'CDINTESTRUTURA', 'req'=>'N'),
            34 => array('name'=>'CDINTESTRUTEXPFIL', 'req'=>'N'),
            35 => array('name'=>'CDINTESTRUTEXP', 'req'=>'N'),
        );
        try {
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $result = $this->apiService->setEstrutura(
                $param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6], $param[7], $param[8], $param[9], $param[10], $param[11], 
                $param[12], $param[13], $param[14], $param[15], $param[16], $param[17], $param[18], $param[19], $param[20], $param[21], $param[22], 
                $param[23], $param[24], $param[25], $param[26], $param[27], $param[28], $param[29], $param[30], $param[31], $param[32], $param[33], $param[34], $param[35]
            );
            $response->addDataSet(new DataSet('setEstrutura', array($result)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    /**
     * Função responsável por criar um quadro de vagas
     *
     * @param Request\Row $request  No request são esperados os campos:
     * Informações obrigatórias:
     * - CDCNPJESTRUT: CNPJ da estrutura
     * - NRTIPOESTRUTURA : Número do tipo de Estrutura conforme descrição abaixo:
     *      1- EMPRESA
     *      2- FILIAL
     *      3- DEPARTAMENTO
     *      4- SETOR
     *      5- CENTRO CUSTO
     *      6- EQUIPE
     *      7- TOMADOR DE SERVICOS
     *      8- REGIÃO
     *      9- PROJETO
     *      10- SINDICATO
     *      11- OBRA
     *      12- PRODUTO
     *      13- PLANO DE SAÚDE
     *      14- ESCOLA
     *      25- LABORATÓRIO
     *      26- FORNECEDOR
     * - CDOCUPACAO: Código de integração da ocupação
     * - QTDLIMITE: Quantidade limite do quadro de vagas
     * - DTINIVALIDADE: Data de início de validade do quadro de vagas no formato DD/MM/YYYY
     * Informações não obrigatórias:
     * - DTFIMVALIDADE: Data fim de validade do quadro de vagas no formato DD/MM/YYYY
     * - CDPROPOSTA: Código da proposta
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [quadro] = Array (
     *          true
     *      )
     *  )
     * )
     */
    public function setQuadroVagas(Request\Row $request, Response $response){
        
        $params = array(
            0 => array('name'=>'CDCNPJESTRUT', 'req'=>'S'),
            1 => array('name'=>'NRTIPOESTRUTURA', 'req'=>'S'),
            2 => array('name'=>'CDOCUPACAO', 'req'=>'S'),
            3 => array('name'=>'QTDLIMITE', 'req'=>'S'),
            4 => array('name'=>'DTINIVALIDADE', 'req'=>'S'),
            5 => array('name'=>'DTFIMVALIDADE', 'req'=>'N'),
            6 => array('name'=>'CDPROPOSTA', 'req'=>'N'),
            7 => array('name'=>'NRESCALATRABM', 'req'=>'N'),
            8 => array('name'=>'CDFILIAL', 'req'=>'N'),
            9 => array('name'=>'CDCENTROCUSTO', 'req'=>'N'),
        );
        try {
            //$this->logService->saveRequestRowData($this->kernel->getHttpRequest());
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $result = $this->apiService->setQuadroVagas(
                $param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6], $param[7], $param[8], $param[9]
            );
            $response->addDataSet(new DataSet('setQuadroVagas', array($result)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    
    /**
     * Função responsável por criar um afastamento por atestado médico.
     *
     * @param Request\Row $request  No request são esperados os campos:
     * Informações obrigatórias:
     * - NRVINCULOM: Filtro por numero de vínculo
     * - DTINICIO: Data de início do afastamento no formato DD/MM/YYYY
     * - QTDIAS: Quantidade de dias de afastamento, caso não seja infromado a data fim do afastamento.
     * - DTFIM: Data fim do afatamento no formato DD/MM/YYYY
     * - QTDIASATESTADOEMPR: Referente ao campo "Qt. Dias Atestado Empresa"
     * - NRMOTIVOAFASTA: Número do motivo de afastamento conforme descrição abaixo:
     *      1 - Acidente de Trabalho 1
     *      2 - Serviço Militar
     *      3 - Licença Maternidade (120 dias)
     *      4 - Doença
     *      5 - Licença Não Remunerada
     *      6 - Licença Motivo de Doença na Família
     *      7 - Licença Mandato Eletivo
     *      8 - Licença Tratamento de Saúde
     *      9 - Licença Paternidade
     *      10 - Mudança de regime jurídico.
     *      11 - Transferência de empregado para outro estabelecimento da mesma empresa.
     *      16 - Transf. de empregado para outra empresa, sem rescisão de contrato de trabalho.
     *      17 - Afastamento temporário por motivo de acidente do trabalho, superior a 15 dias.
     *      18 - Novo afastamento temporário em decorrência do mesmo acidente do trabalho.
     *      20 - Afastamento temporário por motivo de doença, por período superior a 15 dias.
     *      21 - Novo afastamento temporário em decorrência da mesma doença.
     *      22 - Afastamento temporário por motivo de doença, por período igual ou inferior a 15 dias;
     *      23 - Afastamento temporário por motivo de licença-maternidade (120 dias).
     *      24 - Prorrogação do afastamento temporário por motivo de licença-maternidade.
     *      25 - Afastamento temporário por motivo de aborto não criminoso.
     *      27 - Afastamento temporário para prestar serviço militar obrigatório.
     *      28 - Falecimento.
     *      29 - Falecimento motivado por acidente de trabalho.
     *      30 - Aposentadoria por tempo de contribuição ou idade sem continuidade de  vínculo empregatício.
     *      31 - Aposentadoria por invalidez.
     *      32 - Afastamento temporário para exercício de mandato sindical.
     *      33 - Licença com percepção de salário.
     *      34 - Licença sem percepção de salário.
     *      35 - Afastamento por suspensão do contrato de trabalho prevista no art. 476-A da CLT;
     *      36 - Retorno de afastamento temporário por motivo de licença-maternidade, informado pela Q1.
     *      37 - Retorno de afastamento temporário por motivo de licença-maternidade, informado pela Q1.
     *      38 - Retorno de afast. por motivo de acidente do trabalho, sup. a 15 dias, pela O1.
     *      39 - Retorno de afast. por motivo de acidente do trabalho, sup. a 15 dias, pela O1.
     *      40 - Retorno de novo afast. em decorrência do mesmo acidente do trabalho, pela O2;
     *      41 - Retorno do afastamento para prestar serviço militar obrigatório, pela R.
     *      42 - Retorno de afast. por acidente do trabalho, igual ou inferior a 15 dias, pela O3;
     *      43 - Retorno de afast. por motivo de doença, superior a 15 dias, pela P1.
     *      44 - Retorno de licença com percepção de salário, informado pela X1;
     *      45 - Retorno de licença sem percepção de salário, informado pela X2;
     *      46 - Retorno de aposentadoria por invalidez, informado pela movimentação U3;
     *      47 - Retorno do afast. para exercício de mandato sindical,  informado pela W;
     *      48 - Retorno do afast. por motivo de aborto não criminoso,  informado pela Q3;
     *      49 - Retorno da prorrogação do afast. por motivo de licença-maternidade, pela Q2.
     *      50 - Retorno de afast. por motivo de licença-maternidade,  informado pela Q4.
     *      51 - Retorno de afast. por motivo de licença-maternidade,  informado pela Q5.
     *      52 - Retorno de afast. por motivo de licença-maternidade,  informado pela Q6.
     *      53 - Retorno de afast. por motivo de doença, igual ou inferior a 15 dias, pela P3.
     *      54 - Retorno do afast. por suspensão do contrato de trabalho, pela X3.
     *      55 - Despedida por justa causa, pelo empregador
     *      56 - Despedida sem justa causa, pelo empregador
     *      57 - Despedida sem justa causa, pelo empregado
     *      58 - Exercício de mandato eleitoral
     *      59 - Gozo de férias
     *      60 - Cessão de Trabalhador
     *      61 - Cárcere
     *      62 - Doação de Sangue
     *      63 - Acompanhamento Familiar (Médico)
     *      64 - Comparecimento a Justiça/Delegacia
     *      65 - TRATAMENTO ODONTOLOGICO
     *      66 - TRATAMENTO - PSICOLOGA
     *      70 - Afastamento temporário por motivo de acidente de trabalho, igual ou inferior a 15 dias.
     *      72 - Suspensão temporária do contrato de trabalho nos termos da Lei 14.020/2020
     *      78 - Licença remunerada - Lei, liberalidade da empresa ou Acordo/Convenção Coletiva de Trabalho
     * - PROXSITUFUNC: Número da Situação Funcional conforme descrição abaixo:
     *      1 - Atividade Normal
     *      2 - INATIVO - Afastamento com Direito FGTS/INSS (até 30 dias)
     *      3 - Afastamento Acidente Trabalho Superior 15 Dias
     *      4 - Afastamento Serviço Militar
     *      5 - Afastamento Licença Gestante
     *      6 - Afastamento Doença Superior 15 Dias
     *      7 - Afastamento sem Atividade Ocupando Vaga na Empresa
     *      8 - INATIVO - Novo Afastamento Acidente de Trabalho 60 Dias
     *      9 - Novo afastamento temporário em decorrência da mesma doença, dentro de 60 dias;
     *      10 - Falecido
     *      11 - Prorrogação Afastamento Licença Gestante
     *      12 - Atestado Médico
     *      13 - Demitido
     *      14 - Aposentadoria por invalidez
     *      15 - Afastamento doença - 15 dias de responsabilidade do empregador
     *      16 - Afastamento acidente - 15 dias de responsabilidade do empregador
     *      17 - Licença Remunerada - Férias Coletivas
     *      18 - Advertência
     *      100001 - INATIVO - Af.Ac.Trabalho
     *      100002 - INATIVO - Af.Previdência
     *      100003 - Férias
     *      100004 - Licença sem vencimento
     *      100005 - Acidente de Trabalho
     *      100006 - Doença
     *      100007 - Normal não remunerado
     *      100008 - Falta Justificada (Não Remunerada)
     *      100011 - Declaração de comparecimento
     *      100015 - Licenca nao Remunerada
     *      100041 - Suspensao Lei 14.020/2020 (Com Ajuda Compensatória)
     *      100042 - Atestado Médico - Covid19
     *      100043 - Reducao de Jornada Lei 14.020/2020
     *      100044 - Suspensao Lei 14.020/2020 (Sem Ajuda Compensatória)
     *      100051 - Suspensão
     *      100055 - Licença Remunerada
     *      100072 - Falta (Dia)
     * 
     * 
     * @param Response $response    A resposta será no formato de um Dataset (Array)
     * Array (
     *  [dataset] = Array (
     *      [atestadoMedico] = Array (
     *          true
     *      )
     *  )
     * )
     */
    public function setAfastamentoPorAtestadoMedico(Request\Row $request, Response $response){
        
        $params = array(
            0 => array('name'=>'NRVINCULOM', 'req'=>'S'),
            1 => array('name'=>'DTINICIO', 'req'=>'S'),
            2 => array('name'=>'QTDIAS', 'req'=>'N'),
            3 => array('name'=>'DTFIM', 'req'=>'N'),
            4 => array('name'=>'QTDIASATESTADOEMPR', 'req'=>'N'),
            5 => array('name'=>'NRMOTIVOAFASTA', 'req'=>'S'),
            6 => array('name'=>'PROXSITUFUNC', 'req'=>'N'),
        );
        try {
            $row = $request->getRow();
            $param = $this->valida_parametros($params, $row);
            $result = $this->apiService->setAfastamentoPorAtestadoMedico(
                $param[0], $param[1], $param[2], $param[3], $param[4], $param[5], $param[6]
            );
            $response->addDataSet(new DataSet('atestadoMedico', array($result)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }
    /*=================================================================================*/
    
}